/*
 * RCS Info
 * $Id: checkReadXml.c,v 1.16 2006/02/02 08:58:50 r29173 Exp $
 * 
 * Log
 * $Log: checkReadXml.c,v $
 * Revision 1.16  2006/02/02 08:58:50  r29173
 * workaround to possible bug in glib (g_print function)
 *
 * Revision 1.15  2006/01/26 13:39:25  r29173
 * fixed core dump when running admsCheck
 *
 * Revision 1.14  2005/06/17 16:44:27  r29173
 * debug memory allocation with valgrind
 *
 * Revision 1.13  2005/05/09 14:38:30  r29173
 * cleaned-up source directory structure
 *
 * Revision 1.12  2005/05/03 09:35:14  r29173
 * cleaned-up header file dependencies
 *
 * Revision 1.11  2004/11/11 13:45:04  r29173
 * replaced g_message by adms_message_info
 *
 * Revision 1.10  2004/11/02 10:18:02  r29173
 * fixed makefile dependy issues - now 'make distcheck' passes
 *
 * Revision 1.9  2004/10/20 15:33:52  r29173
 * tons of changes
 * 1- introduced element whileloop (marat)
 * 2- reshaped the handling of the messaging system
 * 3- fixed bug in admsCheck/Makefile.am
 * 4- started implementation of implicit adms transforms
 *
 * Revision 1.8  2004/10/18 13:35:40  r29173
 * changed prefix of adms 'rules' from xsl: to adms:
 * renamed 'adms rules' to 'adms transforms'
 * renamed adms transforms adms:defined and adms:undefined to adms:if
 * introduced SiMKiT to testbench
 * improved handling of property 'separator' in adms:join
 * spectre/mica/ads passed
 *
 * Revision 1.6  2004/10/01 19:05:35  marat_yakupov
 * Fixed bugs to allow compilation
 *
 * Revision 1.5  2004/08/21 15:22:08  marat_yakupov
 * Fixed issue with admsCheck
 *
 * Revision 1.4  2004/08/20 13:29:13  r29173
 * cleaned-up i/o file handling
 *
 * Revision 1.3  2004/08/03 12:33:54  r29173
 * import adms-1.21.0 from local CVS
 *
 * Revision 1.1.1.1  2004/05/21 12:20:01  r29173
 * recreated cvs data structure (crashed after revision 1.13.0!)
 *
 * Revision 1.12  2004/03/08 08:24:19  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.11  2004/01/13 11:27:23  r29173
 * fixed bug in dynamic loading of static symbols
 *
 * Revision 1.10  2004/01/09 16:23:46  r29173
 * cleaned-up use of #include
 *
 * Revision 1.9  2004/01/05 11:07:53  r29173
 * change the way admsCheck.xml is downloaded
 *
 * Revision 1.8  2003/12/11 20:14:09  r29173
 * cleaned-up after compilation with CFLAG+=-Wall
 *
 * Revision 1.7  2003/10/22 15:11:36  r29173
 * changed the way script "text2Ccode.pl" handles file name.
 *
 * Revision 1.6  2003/09/01 08:21:49  r29173
 * changed example used to illustrate xml script
 *
 * Revision 1.5  2003/08/28 09:25:48  r29173
 * updated R3 and VBICSELFT
 * added two xsl files from ads interface
 *
 * Revision 1.4  2003/08/27 10:15:27  r29173
 * removed unused files
 *
 * Revision 1.3  2003/08/27 09:28:51  r29173
 * removed unused files
 *
 * Revision 1.2  2003/05/21 14:18:00  r29173
 * add rcs info
 *
 */

#include <admsObject/admsC.h>
#include <admsCheck/admsCheck.h>
#include <admsCheck/admsCheck.xml.c>

#include <admsCheck/basicLibrary.xml.c>
#include <admsCheck/spectreMODULE_dc.ckt.xml.c>
#include <admsCheck/VBICSELFT.va.c>

static void adms_check_read_xml_start_element_handler  (GMarkupParseContext *context,
  const gchar *element_name, const gchar **attribute_names, const gchar **attribute_values,
  gpointer unused, GError **error)
{
  if(!strcmp(element_name,"test"))
  {
    int i = 0;
    while (attribute_names[i])
    {
      char* mydescription=g_strdup(attribute_values[i]);
      if(!strcmp(attribute_names[i],"description"))
      {
        adms_message_info ("%s\n",mydescription);
      }
      free (mydescription);
      ++i;
    }
  }
  else if(!strcmp(element_name,"instruction"))
  {
    int i = 0;
    while (attribute_names[i])
    {
      char* mydescription=g_strdup(attribute_values[i]);
      if(!strcmp(attribute_names[i],"description"))
      {
        adms_message_info ("%s\n",mydescription);
      }
      free (mydescription);
      ++i;
    }
  }
  else if(!strcmp(element_name,"data"))
  {
    int i = 0;
    while (attribute_names[i])
    {
      if(!strcmp(attribute_names[i],"savefilename"))
      {
        char* fileName=g_strdup(attribute_values[i]);
        adms_message_info (" ... create file: %s\n",fileName);
        adms_file_setfile_output(adms_file_open_write (fileName,__FUNCTION__));
        free (fileName);
      }
      else if(!strcmp(attribute_names[i],"symbolname"))
      {
        char *symbol=g_strdup(attribute_values[i]);
        GModule *mymodule=g_module_open (NULL, G_MODULE_BIND_LAZY);
        char **Symbol=NULL;
        if(!mymodule)
          g_error ("error: %s", g_module_error ());
        if(!g_module_symbol ((GModule *)mymodule, symbol, (gpointer) &Symbol))
          adms_message_fatal ("error: %s", g_module_error ());
        g_print("%s ", *Symbol);
        adms_file_close_output ();
        free (symbol); 
        g_module_close (mymodule);
      }
      ++i;
    }
  }
}

static void adms_check_read_xml_end_element_handler (GMarkupParseContext *context,
                        const gchar         *element_name,
                        gpointer             user_data,
                        GError             **error)
{
}

static void adms_check_read_xml_error_handler (GMarkupParseContext *context,
                        GError              *error,
                        gpointer             user_data)
{
  adms_message_fatal (" %s", error->message);
}

static const GMarkupParser parser = {
  adms_check_read_xml_start_element_handler,
  adms_check_read_xml_end_element_handler,
  NULL,
  NULL,
  adms_check_read_xml_error_handler
};

int adms_check_read_xml (void)
{
  GError *error = NULL;
  GMarkupParseContext *context;
  
  g_assert (admsCheck_xml);

  if(!g_module_supported ())
    g_error ("gmodule not supported");

  context = g_markup_parse_context_new (&parser, 0, NULL, NULL);

  if(!g_markup_parse_context_parse (context, admsCheck_xml, strlen(admsCheck_xml), &error))
  {
    g_markup_parse_context_free (context);
    adms_message_fatal ("%s", error->message);
    return 1;
  }

  if(!g_markup_parse_context_end_parse (context, NULL))
  {
    g_markup_parse_context_free (context);
    adms_message_fatal ("%s", error->message);
    return 1;
  }

  g_markup_parse_context_free (context);

  return 0;
}
