/*
 * RCS Info
 * $Id: admsPreprocessor.c,v 1.15 2005/11/08 14:07:59 r29173 Exp $
 * 
 * Log
 * $Log: admsPreprocessor.c,v $
 * Revision 1.15  2005/11/08 14:07:59  r29173
 * fixed bug (skipp_text becomes a list)
 *
 * Revision 1.14  2005/06/21 08:02:28  r29173
 * renamed adms_set_admsmain to adms_valueto_admsmain
 *
 * Revision 1.13  2005/06/17 16:44:24  r29173
 * debug memory allocation with valgrind
 *
 * Revision 1.12  2005/05/09 14:38:30  r29173
 * cleaned-up source directory structure
 *
 * Revision 1.11  2005/05/03 09:35:15  r29173
 * cleaned-up header file dependencies
 *
 * Revision 1.10  2005/05/03 07:58:20  r29173
 * moved admsNS into admsFile
 *
 * Revision 1.9  2004/11/19 12:29:56  r29173
 * renamed _C into CAT
 * saved argc, argv into admsmain element
 * added adms:setenv transform
 *
 * Revision 1.8  2004/11/01 09:37:36  r29173
 * changed messaging macros (avoid useless computation of arguments)
 * improved the coding of transforms in xmlProgram.c
 *
 * Revision 1.7  2004/10/22 17:18:55  marat_yakupov
 * Fixed issue to allow compilation and prevent core dumps
 *
 * Revision 1.6  2004/10/20 15:33:52  r29173
 * tons of changes
 * 1- introduced element whileloop (marat)
 * 2- reshaped the handling of the messaging system
 * 3- fixed bug in admsCheck/Makefile.am
 * 4- started implementation of implicit adms transforms
 *
 * Revision 1.5  2004/08/19 16:45:47  r29173
 * cleaned-up i/o file handling
 *
 * Revision 1.4  2004/08/16 10:52:29  r29173
 * changed handling of adms_message_verbose (created adms_message_debug)
 *
 * Revision 1.3  2004/08/03 12:33:55  r29173
 * import adms-1.21.0 from local CVS
 *
 * Revision 1.3  2004/08/03 10:05:46  r29173
 * removed superelement from admsgui
 * fixed some warnings issued during compilation
 *
 * Revision 1.2  2004/07/09 14:39:11  r29173
 * removed data structure ns
 *
 * Revision 1.1.1.1  2004/05/21 12:20:01  r29173
 * recreated cvs data structure (crashed after revision 1.13.0!)
 *
 * Revision 1.13  2004/03/08 13:58:06  r29173
 * all code lower-cased
 *
 * Revision 1.12  2004/02/14 22:46:48  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.11  2004/02/13 14:28:39  r29173
 * started implementing code builder in yacc parser
 *
 * Revision 1.10  2004/01/16 10:31:25  r29173
 * added YYDEBUG=1
 *
 * Revision 1.9  2004/01/13 11:25:52  r29173
 * added adms_message_usage
 *
 * Revision 1.8  2004/01/09 16:23:47  r29173
 * cleaned-up use of #include
 *
 * Revision 1.7  2004/01/06 12:35:28  r29173
 * fixed the use of globals: input file and output file
 *
 * Revision 1.6  2004/01/05 11:38:48  r29173
 * removed unused code
 *
 * Revision 1.5  2003/12/12 14:33:26  r29173
 * changed construct (a!=NULL) into (a) or (a)?1:0
 *
 * Revision 1.4  2003/12/11 20:14:07  r29173
 * cleaned-up after compilation with CFLAG+=-Wall
 *
 * Revision 1.3  2003/12/11 16:01:53  r29173
 * changed prefix [epst]_adms_ to [epst]_
 *
 * Revision 1.2  2003/05/21 14:18:01  r29173
 * add rcs info
 *
 */


#include <admsPreprocessor/admsPreprocessor.h>
#include <admsObject/admsC.h>

static int admspreprocessor_init (FILE *ifh)
  {
  
    gboolean error;

    g_return_val_if_fail (ifh, TRUE);
  
    adms_preprocessor_setfile_input (ifh);

    error = adms_preprocessor_parse();

    return error;
  }

_t_file_usage_static (adms_preprocessor_usage)
{
  adms_message_usage ("%s [options] source\n",g_get_prgname())
  adms_message_usage ("source: verilog-ams source code\n")
  adms_message_usage ("-D NAME: predefine NAME as a macro, with definition `1'\n")
  adms_message_usage ("-D NAME=DEFINITION: predefine NAME as a macro, with definition DEFINITION\n")
  adms_message_usage ("-I DIR: Add the directory DIR to search path for header files\n")
}
int main(const int argc, const char **argv)
{
  GSList *fileNameList;
  char *firstfileName;
  FILE *ifh;
  g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL);
  if (getenv("adms_preprocessor_debug"))
    adms_preprocessor_setint_yydebug(1);
  else
    adms_preprocessor_setint_yydebug(0);
  g_set_prgname (g_strconcat(PACKAGE_NAME, "Preprocessor", NULL));
  adms_valueto_admsmain (argc,argv);
  if(adms_file_check_command_line (argc, argv,adms_preprocessor_usage))
    return 0;
  fileNameList = adms_file_get_ifile_from_argv(argc, argv);
  firstfileName = (char*) g_slist_nth_data (fileNameList,0);
  if (firstfileName)
    ifh = adms_file_open_read(firstfileName, "command-line");
  else
  {
    ifh = NULL;
    adms_file_print_usage (adms_preprocessor_usage);
  }
  if (ifh)
  {
    p_preprocessor scanner;
    scanner = g_new0(t_preprocessor,1);
    scanner->cur_line_position = 1;
    scanner->cur_char_position = 1;
    scanner->fid = ifh;
    scanner->filename = firstfileName;
    scanner->buffer = NULL;
    adms_preprocessor_new ();
    adms_preprocessor_get_main()->cr_scanner = scanner;
    adms_preprocessor_get_main()->cr_filename = firstfileName;
    adms_preprocessor_get_main()->skipp_text = NULL;
    adms_preprocessor_get_main()->includePath = adms_file_get_ipath_from_argv(argc, argv);
    adms_preprocessor_get_define_from_argv(argc, argv);
    adms_preprocessor_define_add_default("insideADMS");
    if(!admspreprocessor_init (ifh))
    {
      char *header = g_strdup_printf("# 1 \"%s\"\n",adms_preprocessor_get_main()->cr_scanner->filename);
      g_print(header);
      if(adms_preprocessor_get_main()->error)
        adms_message_fatal ("%i Error(s) found during preprocessing\n", adms_preprocessor_get_main()->error)
      else
        adms_message_verbose ("No error found during preprocessing\n")
      adms_preprocessor_get_main()->Text = g_slist_reverse (adms_preprocessor_get_main()->Text);
      adms_preprocessor_text_list_printf(adms_preprocessor_get_main()->Text);
      free(adms_preprocessor_get_main()->cr_scanner);
    }
    else
      adms_message_fatal ("Error found during preprocessing\n")
  }

  return adms_preprocessor_get_main()?adms_preprocessor_get_main()->error:0;
}
