%{

#include <admsVeriloga/admsVeriloga.h>
#include <admsObject/objectLexval.h>
#include <admsVeriloga/verilogaYacc.h>

#undef win32_interface
#if defined(WIN32)
#  if defined(_inside_admsVeriloga)
#    define win32_interface __declspec(dllexport)
#  else
#    define win32_interface __declspec(dllimport)
#  endif
#else
#  define win32_interface
#endif

static int adms_strtointeger (const char *mystr)
{
  int val;
  errno=0;
  val=(int)strtol(mystr,NULL,10);
  if(errno)
    adms_message_fatal ("%s: strtoint conversion failed\n", mystr)
  return val;
}
static char *adms_veriloga_flex_strstrip(const char *mystring)
{
  if(mystring)
  {
    int mynewlen=strlen(mystring)-2;
    if(mynewlen>=0)
    {
      char *mynewstring=(char *)malloc(mynewlen+1);
      if(mynewstring)
      {
        strncpy(mynewstring,mystring+1,mynewlen);
        mynewstring[mynewlen]= '\0';
      }
      return mynewstring;
    }
  }
  return NULL;
}
void adms_veriloga_setfile_input (FILE *ifile)
{
  yyin=ifile;
}
void adms_veriloga_error (const char *s)
{
  adms_message_fatal("%s: during lexical analysis %s at line %i -- see '%s'\n",
    adms_admsmain_valueof_filename(adms_get_admsmain()),s,adms_admsmain_valueof_curline(adms_get_admsmain ()),adms_veriloga_text)
}

#define POS_UPDATE \
{ \
  adms_message_verbose("%s:%i\n",adms_admsmain_valueof_filename(adms_get_admsmain()),adms_admsmain_valueof_fpos(adms_get_admsmain())); \
  adms_admsmain_valueto_fpos(adms_get_admsmain(),adms_admsmain_valueof_lpos(adms_get_admsmain())+1);\
  adms_admsmain_valueto_lpos(adms_get_admsmain(),adms_admsmain_valueof_lpos(adms_get_admsmain())+yyleng); \
}
#define tkreturn(val,token) \
{ \
  POS_UPDATE \
  adms_veriloga_lval._lexval=adms_lexval_new(\
    val,\
    adms_admsmain_valueof_filename(adms_get_admsmain()),\
    adms_admsmain_valueof_curline(adms_get_admsmain()),\
    adms_admsmain_valueof_fpos(adms_get_admsmain()),\
    adms_admsmain_valueof_lpos(adms_get_admsmain())\
  );\
  return token; \
}
#define TKRETURN(tk) tkreturn(g_strdup(yytext),tk);
#define TKSTRIPPEDRETURN(tk) tkreturn(adms_veriloga_flex_strstrip(yytext),tk);
#ifdef YYDEBUG
extern int yydebug;
#endif

%}

whitespace [ \t\v\f]+
newline    \r?\n
ident      [A-Za-z_][A-Za-z0-9_]*
attribute  [A-Za-z_][A-Za-z0-9_:.]*
anystring  \"[^\"]*\"
b8_int     0[0-9]*
b10_uint   [1-9][0-9]*
b16_int    0[xX][0-9A-Fa-f]+
float      [0-9]*\.[0-9]+([eE][+-]?[0-9]+)?|[0-9]+\.?([eE][+-]?[0-9]+)?
char       \'[^\']*[\'\n]

%x insideAttribute 
%x insidePragma 

%%

{newline} {
  adms_admsmain_valueto_curline(adms_get_admsmain(),adms_admsmain_valueof_curline(adms_get_admsmain())+1);
  adms_admsmain_valueto_lpos(adms_get_admsmain(),0);
  adms_admsmain_valueto_fpos(adms_get_admsmain(),0);
}

^#{whitespace} {BEGIN(insidePragma);}
{whitespace} {POS_UPDATE}

<insidePragma>{
{anystring} {adms_admsmain_valueto_filename(adms_get_admsmain(),adms_veriloga_flex_strstrip(yytext));}
{b10_uint} {
    adms_admsmain_valueto_curline(adms_get_admsmain(),adms_strtointeger(yytext));
    adms_admsmain_valueto_lpos(adms_get_admsmain(),0);
    adms_admsmain_valueto_fpos(adms_get_admsmain(),0);
}
{newline} {
  BEGIN(INITIAL);
  adms_admsmain_valueto_lpos(adms_get_admsmain(),0);
  adms_admsmain_valueto_fpos(adms_get_admsmain(),0);
}
{whitespace} {POS_UPDATE}
. {
  adms_message_fatal("[%s:%i]: inside pragma declaration unexpected character at [%s]\n",
    adms_admsmain_valueof_filename (adms_get_admsmain ()),
    adms_admsmain_valueof_curline (adms_get_admsmain ()),
    yytext)
}
}

{anystring} TKSTRIPPEDRETURN(tk_anystring)

\(\* {BEGIN(insideAttribute); TKRETURN(tk_beginattribute)}

<insideAttribute>{
\*\) {
  BEGIN(INITIAL);
  TKRETURN(tk_endattribute)
}
{attribute} TKRETURN(tk_ident)
= TKRETURN(yytext[0])
{anystring} TKSTRIPPEDRETURN(tk_anystring)
{whitespace} {POS_UPDATE}
{newline} {
  adms_admsmain_valueto_curline(adms_get_admsmain(),adms_admsmain_valueof_curline(adms_get_admsmain())+1);
  adms_admsmain_valueto_lpos(adms_get_admsmain(),0);
  adms_admsmain_valueto_fpos(adms_get_admsmain(),0);
}
. {
  adms_message_fatal("[%s:%i]: inside verilogams attribute declaration unexpected character at [%s]\n",
    adms_admsmain_valueof_filename (adms_get_admsmain ()),
    adms_admsmain_valueof_curline (adms_get_admsmain ()),
    yytext)
}
}

parameter TKRETURN(tk_parameter)
aliasparameter TKRETURN(tk_aliasparameter)
module TKRETURN(tk_module)
endmodule TKRETURN(tk_endmodule)
function TKRETURN(tk_function)
endfunction TKRETURN(tk_endfunction)
discipline TKRETURN(tk_discipline)
potential TKRETURN(tk_potential)
flow TKRETURN(tk_flow)
ground TKRETURN(tk_ground)
enddiscipline TKRETURN(tk_enddiscipline)
nature TKRETURN(tk_nature)
endnature TKRETURN(tk_endnature)
input TKRETURN(tk_input)
output TKRETURN(tk_output)
inout TKRETURN(tk_inout)
branch TKRETURN(tk_branch)
analog TKRETURN(tk_analog)
begin TKRETURN(tk_begin)
end TKRETURN(tk_end)
if TKRETURN(tk_if)
while TKRETURN(tk_while)
case TKRETURN(tk_case)
endcase TKRETURN(tk_endcase)
default TKRETURN(tk_default)
for TKRETURN(tk_for)
else TKRETURN(tk_else)
integer TKRETURN(tk_integer)
real TKRETURN(tk_real)
string TKRETURN(tk_string)
from TKRETURN(tk_from)
exclude TKRETURN(tk_exclude)
inf TKRETURN(tk_inf)
INF TKRETURN(tk_inf)

{ident} TKRETURN(tk_ident)

\>\> TKRETURN(tk_op_shr)
\<\< TKRETURN(tk_op_shl)
\&\& TKRETURN(tk_and)
\|\| TKRETURN(tk_or)
\^\~ TKRETURN(tk_bitwise_equr)

\${ident} TKRETURN(tk_dollar_ident)
{char} TKSTRIPPEDRETURN(tk_char)
{b8_int} TKRETURN(tk_number)
{b10_uint} TKRETURN(tk_number)
{b16_int} TKRETURN(tk_number)
{float} TKRETURN(tk_number)

. TKRETURN(yytext[0])

%%

int yywrap (void) {return 1;}
