#!perl -W

use warnings;

my $top_srcdir=shift @ARGV;
die if not $top_srcdir;
my$BUILDER="code automatically created by $0";
my%elementHeader;

require "$top_srcdir/scripts/adms_build.libpl";

sub buildsymbolname
{
  my$attribute=shift;
  my$method=shift;
  my$attribute_name=&attribute_name($attribute);
  my$element_name=&element_name($attribute);
  return"adms_${element_name}_${method}_${attribute_name}";
}
sub buildsupersymbolname
{
  my$element=shift;
  my$parent=shift;
  my$method=shift;
  my$attribute_name=$parent->getAttribute("name");
  my$element_name=$element->getAttribute("name");
  return"adms_${element_name}_${method}_${attribute_name}";
}
my%buildsymbolvalue=(
size=>"enumeration",
datatypename=>"enumeration",
datatypefamily=>"enumeration",
valueof=>"method",
valueto=>"method",
isdefault=>"method",
fromstring=>"method",
tostring=>"method"
);
sub buildsymbolvalue
{
  my$attribute=shift;
  my$property=shift;
  if($buildsymbolvalue{$property} eq "string")
  {
    my$val="\"".&{"attribute_$property"}($attribute)."\"";
    return$val;
  }
  elsif($buildsymbolvalue{$property} eq "enumeration")
  {
    my$val="adms_admst_enumeration_".$property."_".&{"attribute_$property"}($attribute);
    $val=~s/\*/_star/;
    return$val;
  }
  elsif($buildsymbolvalue{$property} eq "method")
  {
    return "".buildsymbolname($attribute,$property)."_impl";
  }
  else
   {die;}
}
sub buildsymbolvalueBis
{
  my$attribute=shift;
  my$property=shift;
  if($buildsymbolvalue{$property} eq "string")
  {
    my$val="\"".&{"attribute_$property"}($attribute)."\"";
    return$val;
  }
  elsif($buildsymbolvalue{$property} eq "enumeration")
  {
    my$val="adms_admst_enumeration_".$property."_".&{"attribute_$property"}($attribute);
    $val=~s/\*/_star/;
    return$val;
  }
  elsif($buildsymbolvalue{$property} eq "method")
  {
    return &buildsymbolname($attribute,$property);
  }
  else
   {die;}
}

# implement attribute_datatypefamily/attribute_datatypename
sub attribute_datatypenameImpl
{
  my$attribute=shift;
  my$element_name=&element_name($attribute);
  my$attribute_datatypefamily=&attribute_datatypefamily($attribute);
  my$attribute_name=&attribute_name($attribute);
  my$attribute_datatypename=&attribute_datatypename($attribute);
  my$attrTypeImpl;
  if($attribute_datatypefamily eq "basictype")
  {
    if($attribute_datatypename eq "integer") {$attrTypeImpl="int"}
    elsif($attribute_datatypename eq "enumeration") {$attrTypeImpl="e_${element_name}_$attribute_name"}
    elsif($attribute_datatypename eq "real") {$attrTypeImpl="p_real"}
    elsif($attribute_datatypename eq "string") {$attrTypeImpl="char*"}
  }
  elsif($attribute_datatypefamily eq "reference")
  {
    if($attribute_datatypename eq "*") {$attrTypeImpl="gpointer"}
    else{$attrTypeImpl="p_$attribute_datatypename"}
  }
  return $attrTypeImpl;
}
sub containerTypeImpl
{
  my$attribute=shift;
  return (&attribute_size($attribute) eq "list")?"GSList*":&attribute_datatypenameImpl($attribute);
}
sub containerTypeImplBis
{
  my$attribute=shift;
  return (&attribute_size($attribute) eq "list")?"GSList*":&attribute_datatypenameImpl($attribute);
}

# build method templates for each attribute 
my$FUNCPREFIX="adms_ELTNAME_list_ATTRNAME";
my$LISTNAME="adms_ELTNAME_valueof_ATTRNAME(mymyELTNAME)";
my$templates={
any_prepend=>qq#
  H win32_interface gpointer ${FUNCPREFIX}_prepend (p_ELTNAME mymyELTNAME,gpointer mydata);
  C gpointer ${FUNCPREFIX}_prepend (p_ELTNAME mymyELTNAME,gpointer myany)
  C {
  C   qpush(&$LISTNAME,myany);
  C   return myany;
  C }
  C #,
reference_prepend_by_id=>qq#
  H win32_interface ATTRCONTAINER ${FUNCPREFIX}_prepend_by_id (p_ELTNAME mymyELTNAME,ATTRID_H);
  C ATTRCONTAINER
  C ${FUNCPREFIX}_prepend_by_id (p_ELTNAME mymyELTNAME,ATTRID_H)
  C {
  C   ATTRCONTAINER refATTRNAME=adms_ATTRVAL_new(ATTRID_C);
  C   qpush(&$LISTNAME,refATTRNAME);
  C   return refATTRNAME;
  C }
  C #,
reference_prepend_by_id_once_or_ignore=>qq#
  H win32_interface ATTRCONTAINER ${FUNCPREFIX}_prepend_by_id_once_or_ignore (p_ELTNAME mymyELTNAME,ATTRID_H);
  C ATTRCONTAINER
  C ${FUNCPREFIX}_prepend_by_id_once_or_ignore (p_ELTNAME mymyELTNAME,ATTRID_H)
  C {
  C   GSList* list=$LISTNAME;
  C   ATTRCONTAINER refATTRNAME=adms_ATTRVAL_new(ATTRID_C);
  C   while(list)
  C   {
  C     if(!adms_ATTRVAL_cmp(list->data, refATTRNAME))
  C     {
  C       free (refATTRNAME);
  C       return list->data;
  C     }
  C     list = list->next;
  C   }
  C   qpush(&$LISTNAME,refATTRNAME);
  C   return refATTRNAME;
  C }
  C #,
reference_prepend_by_id_once_or_passthrough=>qq#
  H win32_interface ATTRCONTAINER ${FUNCPREFIX}_prepend_by_id_once_or_ignore (p_ELTNAME mymyELTNAME,ATTRID_H);
  C ATTRCONTAINER
  C ${FUNCPREFIX}_prepend_by_id_once_or_ignore (p_ELTNAME mymyELTNAME,ATTRID_H)
  C {
  C   GSList* list=$LISTNAME;
  C   ATTRCONTAINER refATTRNAME=adms_ATTRVAL_new(ATTRID_C);
  C   qpush(&$LISTNAME,refATTRNAME);
  C   return refATTRNAME;
  C }
  C #,
reference_prepend_by_id_once_or_abort=>qq#
  H win32_interface ATTRCONTAINER ${FUNCPREFIX}_prepend_by_id_once_or_abort (p_ELTNAME mymyELTNAME,ATTRID_H);
  C ATTRCONTAINER
  C ${FUNCPREFIX}_prepend_by_id_once_or_abort (p_ELTNAME mymyELTNAME,ATTRID_H)
  C {
  C   GSList* list=$LISTNAME;
  C   ATTRCONTAINER refATTRNAME=adms_ATTRVAL_new(ATTRID_C);
  C   while(list)
  C   {
  C     if (!adms_ATTRVAL_cmp(list->data,refATTRNAME))
  C     {
  C       adms_message_fatal ("ELTNAME=[%s] ATTRNAME=[%s] already defined\\n",
  C         adms_ELTNAME_getuid(mymyELTNAME),adms_ATTRVAL_getuid(refATTRNAME))
  C       free (refATTRNAME);
  C     }
  C     list = list->next;
  C   }
  C   qpush(&$LISTNAME,refATTRNAME);
  C   return refATTRNAME;
  C }
  C #,
reference_lookup_by_id=>qq#
  H win32_interface ATTRCONTAINER ${FUNCPREFIX}_lookup_by_id (p_ELTNAME mymyELTNAME,ATTRID_H);
  C ATTRCONTAINER
  C ${FUNCPREFIX}_lookup_by_id (p_ELTNAME mymyELTNAME,ATTRID_H)
  C {
  C   GSList* list=$LISTNAME;
  C   ATTRCONTAINER refATTRNAME=adms_ATTRVAL_new(ATTRID_C);
  C   while(list)
  C   {
  C     if (!adms_ATTRVAL_cmp(list->data,refATTRNAME))
  C     {
  C       free (refATTRNAME);
  C       return list->data;
  C     }
  C     list = list->next;
  C   }
  C   free (refATTRNAME);
  C   return NULL;
  C }
  C #,
reference_prepend_once_or_ignore=>qq#
  H win32_interface void ${FUNCPREFIX}_prepend_once_or_ignore (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME);
  C void
  C ${FUNCPREFIX}_prepend_once_or_ignore (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME)
  C {
  C   GSList* list=$LISTNAME;
  C   while(list)
  C   {
  C     if (!adms_ATTRVAL_cmp (list->data,myATTRNAME))
  C       return;
  C     list = list->next;
  C   }
  C   qpush(&$LISTNAME,myATTRNAME);
  C }
  C #,
reference_prepend_once_or_abort=>qq#
  H win32_interface void ${FUNCPREFIX}_prepend_once_or_abort (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME);
  C void
  C ${FUNCPREFIX}_prepend_once_or_abort (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME)
  C {
  C   GSList* list=$LISTNAME;
  C   while(list)
  C   {
  C     if (!adms_ATTRVAL_cmp (list->data,myATTRNAME))
  C     {
  C       adms_message_fatal ("ELTNAME=[%s] ATTRNAME=[%s] already defined\\n",
  C         adms_ELTNAME_getuid(mymyELTNAME),adms_ATTRVAL_getuid(myATTRNAME))
  C       return;
  C     }
  C     list = list->next;
  C   }
  C   qpush(&$LISTNAME,myATTRNAME);
  C }
  C #,
reference_prepend_once_or_passthrough=>qq#
  H win32_interface void ${FUNCPREFIX}_prepend (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME);
  C void
  C ${FUNCPREFIX}_prepend (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME)
  C {
  C   qpush(&$LISTNAME,myATTRNAME);
  C }
  C #,
basictype_prepend_once_or_ignore=>qq#
  H win32_interface void ${FUNCPREFIX}_prepend_once_or_ignore (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME);
  C void
  C ${FUNCPREFIX}_prepend_once_or_ignore (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME)
  C {
  C   if(!g_slist_find($LISTNAME,myATTRNAME))
  C     qpush(&$LISTNAME,myATTRNAME);
  C }
  C #,
basictype_prepend_once_or_passthrough=>qq#
  H win32_interface void ${FUNCPREFIX}_prepend (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME);
  C void
  C ${FUNCPREFIX}_prepend (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME)
  C {
  C   qpush(&$LISTNAME,myATTRNAME);
  C }
  C #,
basictype_prepend_once_or_abort=>qq#
  H win32_interface void ${FUNCPREFIX}_prepend_once_or_abort (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME);
  C void
  C ${FUNCPREFIX}_prepend_once_or_abort (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME)
  C {
  C   if(!g_slist_find($LISTNAME,myATTRNAME))
  C     qpush(&$LISTNAME,myATTRNAME);
  C   else
  C     adms_message_fatal ("ELTNAME=[%s] ATTRNAME=[%s] already defined\\n",adms_ELTNAME_getuid(mymyELTNAME),myATTRNAME)
  C }
  C #,
basictype_prepend_by_id=>qq#
  H win32_interface void ${FUNCPREFIX}_prepend_by_id (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME);
  C void
  C ${FUNCPREFIX}_prepend_by_id (p_ELTNAME mymyELTNAME,ATTRCONTAINER myATTRNAME)
  C {
  C   qpush(&$LISTNAME,myATTRNAME);
  C }
  C #
};

my$admsParser=XML::LibXML->new->parse_file ("$top_srcdir/adms.xml");

my($myElementArray,$myElementHash)=&parserXml($admsParser);
foreach(@$myElementArray) {&mkNew($_);&mkNewimpl($_);&mkFree($_);}
foreach(@$myElementArray) {&mkCode($_)}
foreach("") {&mkCode($_)} #for objectClass

#template for file naming: module -> objectModule
sub makeFileName
{
  my$name=shift;
  $name=~s/^(.)/object\U$1/;
  return $name;
}

#template for macro adms_..._free
sub mkFree ()
{
  my$element=$_;
  my@attribute=&element_attribute($element);
  my$elementName=$element->getAttribute("name");
  my$Hdec;
  my$Cdec;
  my@arguments;
  my$i=0;
  $FREE{$elementName}{H}.="#define adms_${elementName}_free_debug(item)\\\n";
  $FREE{$elementName}{H}.="  adms_adms(item)->_freed=adms_adms_enumeration_freed_yes;\n";
  $FREE{$elementName}{H}.="#define adms_${elementName}_free(item)\\\n";
  $FREE{$elementName}{H}.="{\\\n";
  $FREE{$elementName}{H}.="  adms_get_admsmain()->_nbadmstdestroy++;\\\n" if $elementName eq "admst";
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_datatypefamily=&attribute_datatypefamily($_);
    my$attribute_size=&attribute_size($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    if($attribute_size eq "scalar")
    {
      if(($attribute_datatypefamily eq "basictype")&&($attribute_datatypename eq "string"))
      {
        $FREE{$elementName}{H}.="  free(item->_${attribute_name});\\\n";
      }
    }
    elsif($attribute_size eq "list")
    {
      if(($attribute_datatypefamily eq "basictype")&&($attribute_datatypename eq "string"))
      {
        $FREE{$elementName}{H}.="  {GSList*l=item->_$attribute_name;for(;l;l=l->next) free(l->data);}\\\n";
        $FREE{$elementName}{H}.="  g_slist_free(item->_$attribute_name);\\\n";
      }
      #$FREE{$elementName}{H}.="  g_slist_free(item->_${attribute_name});\\\n";
    }
  }
  $FREE{$elementName}{H}.="  free(item);\\\n";
  $FREE{$elementName}{H}.="}\n";
}
#template for adms_..._new
sub mkNewimpl ()
{
  my$element=$_;
  my@attribute=&element_attribute($element);
  my$elementName=$element->getAttribute("name");
  my$Hdec;
  my$Cdec;
  my@arguments;
  my$i=0;
  $NEWimpl{$elementName}{C}.="p_adms adms_${elementName}_new_impl (p_adms arguments[])\n";
  $NEWimpl{$elementName}{C}.="{\n";
  $NEWimpl{$elementName}{C}.="  p_${elementName} mynew${elementName};\n";
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_datatypefamily=&attribute_datatypefamily($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    my$isuid=&isuid($_);
    if($isuid)
    {
      my$attribute_datatypenameImpl=&attribute_datatypenameImpl($_);
      my$attribute_datatypefamily=&attribute_datatypefamily($_);
      my$attribute_datatypename=&attribute_datatypename($_);
      if(($attribute_datatypefamily eq "basictype")&&($attribute_datatypename eq "string"))
      {$NEWimpl{$elementName}{C}.="  $attribute_datatypenameImpl my${attribute_name}=g_strdup((${attribute_datatypenameImpl})arguments[$i]);\n";}
      else
      {$NEWimpl{$elementName}{C}.="  $attribute_datatypenameImpl my${attribute_name}=(${attribute_datatypenameImpl})arguments[$i];\n";}
      push @arguments,"my${attribute_name}";
      $i++;
    }
  }
  $NEWimpl{$elementName}{C}.="  mynew${elementName}=adms_${elementName}_new(".(join",",@arguments).");\n";
  $NEWimpl{$elementName}{C}.="  return adms_adms(mynew${elementName});\n";
  $NEWimpl{$elementName}{C}.="}\n";
  $NEWimpl{$elementName}{H}.="p_adms adms_${elementName}_new_impl (p_adms arguments[])";
}
sub mkNew ()
{
  my$element=$_;
  my@attribute=&element_attribute($element);
  my$elementName=$element->getAttribute("name");
  my$Hcode;
  my$Ccode;
  my$args;
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_datatypefamily=&attribute_datatypefamily($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    my$isuid=&isuid($_);
    if($isuid)
    {
      my$attribute_datatypenameImpl=&attribute_datatypenameImpl($_);
      push @$Hcode,"$attribute_datatypenameImpl my$attribute_name";
      push @$Ccode,"my${attribute_name}";
      push @$args,"my${attribute_name}";
    }
  }
  $NEW{$elementName}{Cdec}=(join ",",@$Ccode);
  $NEW{$elementName}{Hdec}=(join ",",@$Hcode);
  $NEW{$elementName}{dec}= "p_$elementName adms_${elementName}_new_1 (".($NEW{${elementName}}{Hdec}).")";
  #memorydebug $NEW{$elementName}{dec}= "p_$elementName adms_${elementName}_new_1 (".($NEW{${elementName}}{Hdec}).",int myline,char* myfile)";
  $NEW{$elementName}{args}=join",",@$args;
}
sub mkCode ()
{
  my$element=shift;
  my$elementName=$element?$element->getAttribute("name"):"class";
  my @C;
  unlink "${top_srcdir}/admsObject/".makeFileName($elementName).".c";
  unlink "${top_srcdir}/admsObject/".makeFileName($elementName).".h";
  if($elementName eq "class")
  {
    my @admsH_H;
    push @admsH_H,"\n/* ------- $BUILDER -------------- */\n";
    push @admsH_H,"\n";
    push @admsH_H,"#ifndef object${elementName}_h\n";
    push @admsH_H,"#define object${elementName}_h\n";
    push @admsH_H,"\n";
    push @admsH_H,"#undef win32_interface\n";
    push @admsH_H,"#if defined(WIN32)\n";
    push @admsH_H,"#  if defined(_inside_admsObject)\n";
    push @admsH_H,"#    define win32_interface __declspec(dllexport)\n";
    push @admsH_H,"#  else\n";
    push @admsH_H,"#    define win32_interface __declspec(dllimport)\n";
    push @admsH_H,"#  endif\n";
    push @admsH_H,"#else\n";
    push @admsH_H,"#  define win32_interface extern\n";
    push @admsH_H,"#endif\n";
    push @admsH_H,"\n";
    foreach(@$myElementArray)
    {
      my$element=$_;
      my$elementName=$element->getAttribute("name");
      push @admsH_H,"typedef struct s_$elementName  t_$elementName;\n";
      push @admsH_H,"typedef t_$elementName*        p_$elementName;\n";
    }
    foreach(qw(real))
    {
      my$elementName=$_;
      push @admsH_H,"typedef struct s_$elementName  t_$elementName;\n";
      push @admsH_H,"typedef t_$elementName*        p_$elementName;\n";
    }
    push @admsH_H,"struct s_real {\n";
    push @admsH_H,"  double _real;\n";
    push @admsH_H,"};\n";
    my($codeH,$codeC)=&miscellaneous_code;
    push @admsH_H,$codeH;
    push @admsH_H,"#endif /* object${elementName}_h */\n";
    open FILE,">${top_srcdir}/admsObject/admsH.h"; print FILE (join "",@admsH_H); close FILE;

    my@admsC_H;
    push @admsC_H,"\n/* ------- $BUILDER -------------- */\n";
    push @admsC_H,"\n";
    push @admsC_H,"#ifndef adms_h\n";
    push @admsC_H,"#define adms_h\n";
    push @admsC_H,"\n";
    push @admsC_H,"#include <admsObject/admsH.h>\n";
    foreach(@$myElementArray)
    {
      my$element=$_;
      my $elementName=$element->getAttribute("name");
      push @admsC_H,"#include <admsObject/".makeFileName($elementName).".h>\n";
    }
    push @admsC_H,"#endif /* adms_h */\n";
    open FILE,">${top_srcdir}/admsObject/admsC.h"; print FILE (join "",(@admsC_H)); close FILE;
  }
  else
  {
    my @H;
    push @H,"\n/* ------- $BUILDER -------------- */\n";
    push @H,"\n";
    push @H,"#ifndef object${elementName}_h\n";
    push @H,"#define object${elementName}_h\n";
    push @H,"#include <admsObject/admsH.h>\n";
    my$parent=&element_parent($element);
    if($parent)
    {
      my$parentName=$parent->getAttribute ("name");
      push @H,"#include <admsObject/".makeFileName(${parentName}).".h>\n";
    }
    my$hEnum=&gettor_attribute_enumeration ($element,"h");
    push @H,$hEnum if $hEnum;
    push @H,&mkElementStructure($element);
    my@attribute=&element_attribute($element);
    foreach(@attribute)
    {
      my$attribute_name=&attribute_name($_);
      my$attribute_datatypefamily=&attribute_datatypefamily($_);
      my$attribute_datatypename=&attribute_datatypename($_);
      my$isuid=&isuid($_);
      if(($attribute_datatypefamily eq "reference")&&($attribute_datatypename ne "*"))
      {
        push @H,"#include <admsObject/".makeFileName($attribute_datatypename).".h>\n" if not $Halreadyin{$elementName}{$attribute_datatypename};
        $Halreadyin{$elementName}{$attribute_datatypename}=1;
      }
    }
    push @H,"\n";
    push @H,"#undef win32_interface\n";
    push @H,"#if defined(WIN32)\n";
    push @H,"#  if defined(_inside_admsObject)\n";
    push @H,"#    define win32_interface __declspec(dllexport)\n";
    push @H,"#  else\n";
    push @H,"#    define win32_interface __declspec(dllimport)\n";
    push @H,"#  endif\n";
    push @H,"#else\n";
    push @H,"#  define win32_interface extern\n";
    push @H,"#endif\n";
    push @H,"\n";
    push @H,"win32_interface char*adms_${elementName}_getuid (p_$elementName my$elementName);\n";
    push @H,"win32_interface gboolean adms_${elementName}_cmp (p_$elementName my$elementName,p_$elementName ref$elementName);\n";
    push @H,"win32_interface ".$NEW{$elementName}{dec}.";\n" if($elementName ne "adms");
    push @H,$FREE{$elementName}{H};
    push @H,"win32_interface ".$NEWimpl{$elementName}{H}.";\n";
    push @H,"win32_interface p_$elementName adms_${elementName}_impl (void* member);\n";
    push @H,"#define adms_$elementName(member) ((p_$elementName) (member))\n";
    #memorydebug push @H,"#define adms_$elementName(member) adms_${elementName}_impl(member)\n";
    push @H,"#define adms_${elementName}_new adms_${elementName}_new_1\n";
    #memorydebug push @H,"#define adms_${elementName}_new(".$NEW{$elementName}{args}.") adms_${elementName}_new_1(".$NEW{$elementName}{args}.",__LINE__,__FILE__)\n";
    push @H,&mkAttributeValueof($element,"h");
    push @H,&mkAttributeValueto($element,"h");
    push @C,"\n/* ------- $BUILDER -------------- */\n";
    push @C,"#include <admsObject/".makeFileName($elementName).".h>\n";
    push @C,"#include <admsObject/".makeFileName("admsmain").".h>\n";
    push @C,"\n";
    push @C,"
             p_$elementName adms_${elementName}_impl (void* member)
             {
               if(((p_adms)member)->_freed==adms_adms_enumeration_freed_yes)
                 G_BREAKPOINT();
               return (p_$elementName) member;
             }\n";
    push @C,"\n";
    push @C,&mkElementCmp($element);
    push @C,&mkElementNew($element) if($elementName ne "adms");
    if($elementName eq "admsttransform")
    {
      my($codeH,$codeC)=&miscellaneous_code;
      push @C,$codeC;
    }
    push @C,&mkAttributeValueof($element,"c");
    push @C,&mkAttributeValueto($element,"c");
    my$cEnum=&gettor_attribute_enumeration ($element,"c");
    push @C,$cEnum if $cEnum;
    if($elementName eq "admsttransform")
    {
      my($codeH,$codeC)=&mkLink_ADMSXML_TO_ADMSTRANSFORM;
      push @H,$codeH;
      push @C,$codeC;
      ($codeH,$codeC)=&mkGet_UID;
      push @H,$codeH;
      push @C,$codeC;
      ($codeH,$codeC)=&mkGet_NEW;
      push @H,$codeH;
      push @C,$codeC;
      ($codeH,$codeC)=&mkGet_CMP;
      push @H,$codeH;
      push @C,$codeC;
      ($codeH,$codeC)=&mkGet_ALLCHILDREN;
      push @H,$codeH;
      push @C,$codeC;
    }
    push @H,"#endif /* object${elementName}_h */\n";
    open FILE,">${top_srcdir}/admsObject/".makeFileName($elementName).".h"; print FILE (join "",(@H)); close FILE;
    open FILE,">${top_srcdir}/admsObject/".makeFileName($elementName).".c"; print FILE (join "",(@C)); close FILE;
  }
}
sub mkElementStructure ()
{
  my$element=$_;
  my$elementName=$element->getAttribute("name");
  my$structure.="struct s_$elementName {\n";
  my$parent=&element_parent($element);
  if($parent)
  {
    my$parentName=$parent->getAttribute ("name");
    $structure.="  t_$parentName _$parentName;\n";
  }
  my@attribute=&element_attribute($element);
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$containerTypeImpl=&containerTypeImplBis($_);
    $structure.="  $containerTypeImpl "."_$attribute_name;\n";
    $structure.="  int _${attribute_name}_isdefault : 1;\n";
  }
  $structure.="};\n";
  return $structure;
}
sub mkAttributeValueof ()
{
  my$element=shift;
  my$type=shift;
  my$elementName=$element->getAttribute("name");
  my$Hcode;
  my$Ccode;
  my@attribute=&element_attribute($element);
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    my$containerTypeImpl=&containerTypeImpl($_);
    my$valueof=&buildsymbolname($_,"valueof");
    my$isdefault=&buildsymbolname($_,"isdefault");
    $Ccode.="";
    $Hcode.="#define $isdefault($elementName) (adms_$elementName($elementName)->_${attribute_name}_isdefault)\n";
    $Hcode.="#define $valueof($elementName) (adms_$elementName($elementName)->_$attribute_name)\n";
  }
  my$parent=$element;
  while($parent=&element_parent($parent))
  {
    my$parentName=$parent->getAttribute ("name");
    my$attribute_name=$parentName;
    my$containerTypeImpl="gpointer";
    my$valueof=&buildsupersymbolname($element,$parent,"valueof");
    $Hcode.="#define $valueof($elementName) ((p_$parentName)$elementName)\n";
  }
  return $Hcode if($type eq "h");
  return $Ccode if($type eq "c");
}
sub gettor_attribute_enumeration ()
{
  my$element=shift;
  my$type=shift;
  my$elementName=$element->getAttribute("name");
  my$Hcode;
  my$Ccode;
  my@attribute=&element_attribute($element);
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    if($attribute_datatypename eq "enumeration")
    {
      my$attribute_datatypenameImpl=&attribute_datatypenameImpl($_);
      my$evalue=&evalue($_);
      $Hcode.="typedef enum {\n";
      my@Henum;
      my$i=0;
      foreach $v (sort @$evalue)
      {
        push @Henum,"  adms_${elementName}_enumeration_${attribute_name}_${v}";
        $i++;
      }
      $Hcode.=join ",\n", @Henum;
      $Hcode.="\n} $attribute_datatypenameImpl;\n";
      $Ccode.="$attribute_datatypenameImpl\n".&buildsymbolname($_,"fromstring")."_impl (p_admsttransform myadmsttransform,char*token)\n";
      $Ccode.="{\n";
      foreach $v (sort @$evalue)
      {
        my$v_lc=lc $v;
        $Ccode.="  if (!strcmp(token,\"$v\"))\n";
        $Ccode.="    return adms_${elementName}_enumeration_${attribute_name}_${v};\n";
      }
      $Ccode.="  adms_message_fatal_continue (\"select='${elementName}/${attribute_name}': possible values:\\n\");\n";
      $Ccode.="  adms_message_fatal_continue (\"" . join("|",sort @$evalue) ."\\n\");\n";
      $Ccode.="  adms_message_fatal (\"see %s\\n\",adms_admsttransform_getuid(myadmsttransform));\n";
      $Ccode.="}\n";
      $Ccode.="char*\n".&buildsymbolname($_,"tostring")."_impl (p_admsttransform myadmsttransform,$attribute_datatypenameImpl e)\n";
      $Ccode.="{\n";
      foreach $v (sort @$evalue)
      {
        my$v_lc=lc $v;
        $Ccode.="  if (e==adms_${elementName}_enumeration_${attribute_name}_${v})\n";
        $Ccode.="    return \"${v}\";\n";
      }
      $Ccode.="  adms_message_fatal_continue (\"select='${elementName}/${attribute_name}': possible values:\\n\");\n";
      $Ccode.="  adms_message_fatal_continue (\"" . join("|",sort @$evalue) ."\\n\");\n";
      $Ccode.="  adms_message_fatal (\"see %s\\n\",adms_admsttransform_getuid(myadmsttransform));\n";
      $Ccode.="}\n";
      $Ccode.="\n";
      $Hcode.="win32_interface char* ".&buildsymbolname($_,"tostring")."_impl (p_admsttransform myadmsttransform,$attribute_datatypenameImpl e);\n";
      $Hcode.="win32_interface $attribute_datatypenameImpl ".&buildsymbolname($_,"fromstring")."_impl (p_admsttransform myadmsttransform,char*token);\n";
    }
  }
  return $Hcode if($type eq "h");
  return $Ccode if($type eq "c");
}
sub mkAttributeValueto_cb
{
  my$codename=shift;
  my$attribute=shift;
  my$elementName=shift;
  my$attribute_datatypefamily=&attribute_datatypefamily($attribute);
  my$attribute_name=&attribute_name($attribute);
  my$attribute_datatypename=&attribute_datatypename($attribute);
  my$attribute_datatypenameImpl=&attribute_datatypenameImpl($attribute);
  my$code=$templates->{$codename};
  die $codename if not $code;
  $code=~s/ELTNAME/$elementName/g;
  $code=~s/ATTRNAME/$attribute_name/g;
  $code=~s/ATTRVAL/$attribute_datatypename/g;
  $code=~s/ATTRCONTAINER/$attribute_datatypenameImpl/g;
  $code=~s/ATTRID_H/$NEW{$attribute_datatypename}{Hdec}/g;
  $code=~s/ATTRID_C/$NEW{$attribute_datatypename}{Cdec}/g;
  $code=~s/^\n//;
  my$c=$code; $c=~s/\n?\s*H.*(\n|$)//g;
  my$h=$code; $h=~s/\n?\s*C.*(\n|$)//g;
  $c=~s/(\n?)\s*C /$1/g;
  $h=~s/(\n?)\s*H /$1/g;
  return("$h\n",$c);
}
sub mkAttributeValueto ()
{
  my$element=shift;
  my$type=shift;
  my$elementName=$element->getAttribute("name");
  my$Hcode;
  my$Ccode;
  my@attribute=&element_attribute($element);
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_size=&attribute_size($_);
    my$attribute_datatypefamily=&attribute_datatypefamily($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    my$attrOnduplicate=&attrOnduplicate($_);
    my$attribute_onrewrite=&attribute_onrewrite($_);
    my$containerTypeImpl=&containerTypeImpl($_);
    my$valueto=&buildsymbolname($_,"valueto");
    my$valueof=&buildsymbolname($_,"valueof");
    #$Hcode.="#define ${valueto} ${valueto}_impl\n";
    $Hcode.="#define ${valueto}(a,b) ((p_$elementName)a)->_$attribute_name=(b)\n";
    $Hcode.="win32_interface void ${valueto}_impl (p_$elementName my${elementName},${containerTypeImpl} _$attribute_name);\n";
    $Ccode.="\nvoid\n${valueto}_impl (p_$elementName my$elementName,$containerTypeImpl _$attribute_name)\n{\n";
    if($attribute_onrewrite && ($attribute_onrewrite eq "abort"))
    {
      $Ccode.="  if(!my$elementName->_${attribute_name}_isdefault)\n";
      $Ccode.="   adms_message_debug (\"\%s:${attribute_name}: error ... testing [onrewrite=abort]\\n\",\n";
      $Ccode.="     adms_${elementName}_getuid(my$elementName))\n";
    }
    $Ccode.="  (adms_$elementName (my$elementName))->_$attribute_name=_$attribute_name;\n";
    $Ccode.="  (adms_$elementName (my$elementName))->_${attribute_name}_isdefault=0;\n";
    $Ccode.="}\n";
    if($attribute_size eq "list")
    {
      my($h,$c);
      if($attribute_datatypename eq "*")
      {
        ($h,$c)=&mkAttributeValueto_cb("any_prepend",$_,$elementName);
      }
      elsif($attribute_datatypefamily eq "reference")
      {
        ($h,$c)=&mkAttributeValueto_cb("reference_lookup_by_id",$_,$elementName);
        my($H,$C)=&mkAttributeValueto_cb("reference_prepend_by_id",$_,$elementName);
        $c.=$C; $h.=$H;
        while($attrOnduplicate =~ s/(\w+)//)
        {
          my($H,$C)=&mkAttributeValueto_cb("reference_prepend_by_id_once_or_$1",$_,$elementName);
          $c.=$C; $h.=$H;
          ($H,$C)=&mkAttributeValueto_cb("reference_prepend_once_or_$1",$_,$elementName);
          $c.=$C; $h.=$H;
        }
      }
      elsif($attribute_datatypefamily eq "basictype")
      {
        my($H,$C)=&mkAttributeValueto_cb("basictype_prepend_by_id",$_,$elementName);
        $c.=$C; $h.=$H;
        while($attrOnduplicate =~ s/(\w+)//)
        {
          my($H,$C)=&mkAttributeValueto_cb("basictype_prepend_once_or_$1",$_,$elementName);
          $c.=$C; $h.=$H;
        }
      }
      else
        {die;}
      $Hcode.=$h;
      $Ccode.=$c;
    }
  }
  return $Hcode if($type eq "h");
  return $Ccode if($type eq "c");
}
sub mkElementCmp ()
{
  my$element=$_;
  my$elementName=$element->getAttribute("name");
  my$code;
  my$codeItem;
  $code.="gboolean adms_${elementName}_cmp (p_$elementName my${elementName},p_$elementName ref$elementName)\n";
  $code.="{\n";
  $code.="  gboolean status;\n";
  my@attribute=&element_attribute($element);
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_size=&attribute_size($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    my$isuid=&isuid($_);
    if($isuid)
    {
      my$valueof=&buildsymbolname($_,"valueof");
      my$cur="${valueof} (my$elementName)";
      my$ref="${valueof} (ref$elementName)";
      if($attribute_datatypename eq "string")
        {push @$codeItem,"!!strcmp($cur,$ref)";}
      else
        {push @$codeItem,"($cur != $ref)";}
    }
  }
  $code.="  status =\n";
  $code.="  (\n    ";
  $code.=join "\n    ||\n    ",@$codeItem if $codeItem;
  $code.="\n  );\n";
  $code.="  return status;\n";
  $code.="}\n";
  $code="" if !$codeItem;
  return $code;
}
sub mkElementNew ()
{
  my$element=$_;
  my$elementName=$element->getAttribute("name");
  my$code;
  my$codeItem;
  my$codeStringify;
  my$codeStringifyItem;
  my$format;
  my@attribute=&element_attribute($element);
  foreach(@attribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_size=&attribute_size($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    my$isuid=&isuid($_);
    my$attribute_uid_format=&attribute_uid_format($_);
    my$attribute_default=&attribute_default($_);
    $format=$attribute_uid_format||$format;
    my$valueto=&buildsymbolname($_,"valueto");
    if($isuid)
    {
      push @$codeItem,"$valueto (mynew$elementName,my$attribute_name);";
      push @$codeStringify,$_;
    }
    elsif(defined $attribute_default)
    {
      my $defval;
      $defval="mynew$elementName->_${attribute_name}=adms_${elementName}_enumeration_${attribute_name}_${attribute_default};" if $attribute_datatypename eq "enumeration";
      $defval="mynew$elementName->_${attribute_name}=$attribute_default;" if $attribute_datatypename eq "integer";
      if($attribute_datatypename eq "real")
      {
        $defval.="{\n";
        $defval.="     p_real myreal=g_new(t_real,1);\n";
        $defval.="     myreal->_real=$attribute_default;\n";
        $defval.="     (mynew$elementName)->_${attribute_name}=myreal;\n";
        $defval.="  }\n";
      }
      $defval="mynew$elementName->_${attribute_name}=g_strdup(\"$attribute_default\");" if $attribute_datatypename eq "string";
      push @$codeItem,$defval;
    }
    push @$codeItem,"mynew$elementName->_${attribute_name}_isdefault=1;";
  }

  $code.=$NEW{$elementName}{dec}."\n";
  $code.="{\n";
  $code.="  p_$elementName mynew$elementName=(p_$elementName) g_new0(t_$elementName,1);\n";
  #memorydebug $code.="  qpush(&(adms_get_admsmain()->_admstnew),$elementName);\n" if $elementName eq "admst";
  $code.="  adms_get_admsmain()->_nbadmstnew++;\n" if $elementName eq "admst";
  my$parent=&element_parent($element);
  if($parent)
  {
    $code.=&parent_new($element,"mynew$elementName->",$element,$parent);
  }
  $code.="  ";
  $code.=join "\n  ",@$codeItem if $codeItem;
  $code.="\n";
  $code.="  return mynew$elementName;\n";
  $code.="}\n";
  $code.=$NEWimpl{$elementName}{C};
  my$count=1;
  foreach(@$codeStringify)
  {
    my$item;        
    my$attribute_name=&attribute_name($_);
    my$attribute_datatypefamily=&attribute_datatypefamily($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    if($attribute_datatypename eq "string")  {$item="mynew$elementName->_$attribute_name";}
    elsif($attribute_datatypename eq "enumeration") {$item="g_strdup_printf(\"\%i\",mynew$elementName->_$attribute_name)";}
    elsif($attribute_datatypename eq "integer") {$item="g_strdup_printf(\"\%i\",mynew$elementName->_$attribute_name)";}
    elsif($attribute_datatypename eq "*") {$item="g_strdup_printf(\"\%p\",mynew$elementName->_$attribute_name)";}
    elsif($attribute_datatypefamily eq "reference") {$item="adms_${attribute_datatypename}_getuid(mynew$elementName->_$attribute_name)";}
    else{die "attribute_datatypefamily=$attribute_datatypefamily";}
    $format =~ s/\$$count/",$item,"/g;
    push @$codeStringifyItem,$item;
    $count++;
  }
  if($format)
  {
    $format="\"$format\",";
    $format=~s/"",//g;
    $code.="char*adms_${elementName}_getuid (p_$elementName mynew$elementName)\n";
    $code.="{\n";
    $code.="  return g_strconcat(\n";
    $code.="    ${format}\n";
    $code.="  NULL);\n";
    $code.="}\n";
  }
  return $code;
}
sub parent_new
{
  my$leaf=shift;
  my$leafpath=shift;
  my$element=shift;
  my$parent=shift;
  my$code;
  my$parentName=$parent->getAttribute("name");
  my$superElement=$myElementHash->{$parentName};
  my$supersuperElement=&element_parent($superElement);
  if($supersuperElement)
  {
    my$newcode=&parent_new(${leaf},"${leafpath}_$parentName.",$superElement,$supersuperElement);
    $code.=$newcode;
  }
  my@superElementAttribute=&element_attribute($superElement);
  foreach(@superElementAttribute)
  {
    my$attribute_name=&attribute_name($_);
    my$attribute_datatypename=&attribute_datatypename($_);
    my$attribute_default=&attribute_default($_);
    if($attribute_default)
    {
      my $defval;
      if($attribute_default=~/^@/)
      {
        $attribute_default=~s/@//;
        $defval="${leafpath}_$parentName._${attribute_name}=adms_adms_enumeration_datatypename_".${leaf}->getAttribute($attribute_default);
      }
      else
      {
        $defval="${leafpath}_$parentName._${attribute_name}=$attribute_default" if $attribute_datatypename eq "integer";
        if($attribute_datatypename eq "real")
        {
          $defval.="  {\n";
          $defval.="     p_real myreal=g_new(t_real,1);\n";
          $defval.="     myreal->_real=$attribute_default;\n";
          $defval.="     (${leafpath}_$parentName)._${attribute_name}=myreal;\n";
          $defval.="  }\n";
        }
        $defval="${leafpath}_$parentName._${attribute_name}=adms_${parentName}_enumeration_${attribute_name}_${attribute_default}" if $attribute_datatypename eq "enumeration";
        $defval="${leafpath}_$parentName._${attribute_name}=\"$attribute_default\"" if(($attribute_datatypename eq "string")&&(($attribute_default ne "myfile")));
        $defval="${leafpath}_$parentName._${attribute_name}=$attribute_default" if(($attribute_datatypename eq "string")&&(($attribute_default eq "myfile")));
      }
      $code.="  $defval;\n";
    }
  }
  return $code;
}

sub miscellaneous_code
{
  my$codeH;
  my$codeC;
  $codeC.="\n/*-- Miscellaneous routines --*/\n";
  $codeC.="GSList* qpush(GSList** list, gpointer data)\n";
  $codeC.="{\n";
  $codeC.="  g_return_val_if_fail(list, NULL);\n";
  $codeC.="  *list = g_slist_prepend(*list,data);\n";
  $codeC.="  return *list;\n";
  $codeC.="}\n";
  $codeC.="gpointer qpull(GSList** list )\n";
  $codeC.="{\n";
  $codeC.="  GSList* first;\n";
  $codeC.="  gpointer data;\n";
  $codeC.="  first = *list;\n";
  $codeC.="  data = first->data;\n";
  $codeC.="  *list = g_slist_remove_link(*list,first);\n";
  $codeC.="  g_slist_free_1(first);\n";
  $codeC.="  return data;\n";
  $codeC.="}\n";
  $codeC.="p_admsmain admsMain;\n";
  $codeC.="p_admsmain adms_get_admsmain(void)\n";
  $codeC.="{\n";
  $codeC.="  return admsMain;\n";
  $codeC.="}\n";
  $codeC.="void adms_valueto_admsmain(const int argc, const char **argv)\n";
  $codeC.="{ \n";
  $codeC.="  int i;\n";
  $codeC.="  admsMain=adms_admsmain_new (g_strdup(\"admsmain\")); \n";
  $codeC.="  adms_admsmain_valueto_argc (adms_get_admsmain(),argc);\n";
  $codeC.="  for(i=1;i<argc;i++)\n";
  $codeC.="    adms_admsmain_list_argv_prepend_once_or_abort(admsMain,g_strdup(argv[i]));\n";
  $codeC.="}\n";
  $codeC.="\n/*-- Messaging --*/\n";
  $codeC.="\n";
  $codeC.="#define MESSAGE_TEMPLATE_IMPL(name,ofh,isfatal)\\\n";
  $codeC.="_t_message (adms_message_ ## name ##_impl) \\\n";
  $codeC.="{\\\n";
  $codeC.="  va_list args;\\\n";
  $codeC.="  char* string;\\\n";
  $codeC.="  g_return_if_fail (format != NULL);\\\n";
  $codeC.="  va_start (args, format);\\\n";
  $codeC.="  string = g_strdup_vprintf (format, args);\\\n";
  $codeC.="  va_end (args);\\\n";
  $codeC.="  fputs (\"[\" # name \"] \",ofh);\\\n";
  $codeC.="  fputs (string,ofh);\\\n";
  $codeC.="  fflush (ofh);\\\n";
  $codeC.="  free (string);\\\n";
  $codeC.="  if(isfatal)\\\n";
  $codeC.="    if(getenv(\"adms_breakpoint\"))\\\n";
  $codeC.="      G_BREAKPOINT();\\\n";
  $codeC.="    else\\\n";
  $codeC.="      exit(1);\\\n";
  $codeC.="}\\\n";
  $codeC.="_t_message (adms_message_ ## name ##_continue_impl) \\\n";
  $codeC.="{\\\n";
  $codeC.="  va_list args;\\\n";
  $codeC.="  char* string;\\\n";
  $codeC.="  g_return_if_fail (format != NULL);\\\n";
  $codeC.="  va_start (args, format);\\\n";
  $codeC.="  string = g_strdup_vprintf (format, args);\\\n";
  $codeC.="  va_end (args);\\\n";
  $codeC.="  fputs (\"[\" # name \"] \",ofh);\\\n";
  $codeC.="  fputs (string,ofh);\\\n";
  $codeC.="  fflush (ofh);\\\n";
  $codeC.="  free (string);\\\n";
  $codeC.="}\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(info,stdout,0)\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(usage,stdout,0)\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(verbose,stdout,0)\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(debug,stdout,0)\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(hint,stderr,0)\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(warning,stderr,0)\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(obsolete,stderr,0)\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(error,stderr,0)\n";
  $codeC.="MESSAGE_TEMPLATE_IMPL(fatal,stderr,1)\n";
  $codeC.="\n";
  $codeH.="\n/*-- Miscellaneous routines --*/\n";
  $codeH.="#include <config.h>\n";
  $codeH.="#include <assert.h>\n";
  $codeH.="#include <signal.h>\n";
  $codeH.="#include <stdarg.h>\n";
  $codeH.="#include <stdio.h>\n";
  $codeH.="#include <stdlib.h>\n";
  $codeH.="#include <ctype.h>\n";
  $codeH.="#include <string.h>\n";
  $codeH.="#include <errno.h>\n";
  $codeH.="#include <locale.h>\n";
  $codeH.="#include <time.h>\n";
  $codeH.="#include <glib.h>\n";
  $codeH.="#include <gmodule.h>\n";
  $codeH.="\n";
  $codeH.="#ifdef HAVE_DMALLOC\n";
  $codeH.="#include <dmalloc.h>\n";
  $codeH.="#endif\n";
  $codeH.="\n";
  $codeH.="win32_interface GSList* qpush(GSList** list, gpointer data);\n";
  $codeH.="win32_interface gpointer qpull(GSList** list);\n";
  $codeH.="\n";
  $codeH.="typedef void *(*p_anyfunction) (void *arg);\n";
  $codeH.="typedef int (*p_getinteger) (gpointer name);\n";
  $codeH.="typedef char* (*p_valuetostring) (p_admsttransform myadmsttransform, gpointer name);\n";
  $codeH.="typedef int (*p_valueofstring) (p_admsttransform myadmsttransform, gpointer name);\n";
  $codeH.="typedef p_real (*p_getreal) (gpointer name);\n";
  $codeH.="typedef char * (*p_getstring) (gpointer name);\n";
  $codeH.="typedef GSList *(*p_getlist) (gpointer name);\n";
  $codeH.="typedef gpointer *(*p_setany) (gpointer name,gpointer value);\n";
  $codeH.="typedef gpointer *(*p_getany) (gpointer name);\n";
  $codeH.="\n";
  $codeH.="win32_interface p_admsmain adms_get_admsmain(void);\n";
  $codeH.="win32_interface void adms_valueto_admsmain(const int argc, const char **argv);\n";
  $codeH.="\n";
  $codeH.="typedef p_adms (t_new) (p_adms arguments[]);\n";
  $codeH.="typedef p_adms (*p_new) (p_adms arguments[]);\n";
  $codeH.="#define _t_new(function) p_adms (function) (p_adms arguments[])\n";
  $codeH.="\n";
  $codeH.="typedef int (t_cmp) (p_adms myadms, p_adms myadmsref);\n";
  $codeH.="typedef int (*p_cmp) (p_adms myadms, p_adms myadmsref);\n";
  $codeH.="#define _t_cmp(function) int (function) (p_adms myadms, p_adms myadmsref)\n";
  $codeH.="\n/*-- Messaging --*/\n";
  $codeH.="\n";
  $codeH.="typedef void (t_message) (const char*format, ...);\n";
  $codeH.="typedef void (*p_message) (const char*format, ...);\n";
  $codeH.="#define _t_message(function)  void (function) (const char*format, ...)\n";
  $codeH.="win32_interface t_message adms_message_info_impl;\n";
  $codeH.="win32_interface t_message adms_message_usage_impl;\n";
  $codeH.="win32_interface t_message adms_message_verbose_impl;\n";
  $codeH.="win32_interface t_message adms_message_debug_impl;\n";
  $codeH.="win32_interface t_message adms_message_hint_impl;\n";
  $codeH.="win32_interface t_message adms_message_warning_impl;\n";
  $codeH.="win32_interface t_message adms_message_obsolete_impl;\n";
  $codeH.="win32_interface t_message adms_message_error_impl;\n";
  $codeH.="win32_interface t_message adms_message_fatal_impl;\n";
  $codeH.="win32_interface t_message adms_message_info_continue_impl;\n";
  $codeH.="win32_interface t_message adms_message_usage_continue_impl;\n";
  $codeH.="win32_interface t_message adms_message_verbose_continue_impl;\n";
  $codeH.="win32_interface t_message adms_message_debug_continue_impl;\n";
  $codeH.="win32_interface t_message adms_message_hint_continue_impl;\n";
  $codeH.="win32_interface t_message adms_message_warning_continue_impl;\n";
  $codeH.="win32_interface t_message adms_message_obsolete_continue_impl;\n";
  $codeH.="win32_interface t_message adms_message_error_continue_impl;\n";
  $codeH.="win32_interface t_message adms_message_fatal_continue_impl;\n";
  $codeH.="#define adms_message_info(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_info (adms_get_admsmain())==adms_admsmain_enumeration_info_yes))\\\n";
  $codeH.="    adms_message_info_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_usage(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_usage (adms_get_admsmain())==adms_admsmain_enumeration_usage_yes))\\\n";
  $codeH.="    adms_message_usage_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_verbose(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_verbose (adms_get_admsmain())==adms_admsmain_enumeration_verbose_yes))\\\n";
  $codeH.="    adms_message_verbose_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_debug(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_debug (adms_get_admsmain())==adms_admsmain_enumeration_debug_yes))\\\n";
  $codeH.="    adms_message_debug_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_hint(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_hint (adms_get_admsmain())==adms_admsmain_enumeration_hint_yes))\\\n";
  $codeH.="    adms_message_hint_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_warning(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_warning (adms_get_admsmain())==adms_admsmain_enumeration_warning_yes))\\\n";
  $codeH.="    adms_message_warning_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_obsolete(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_obsolete (adms_get_admsmain())==adms_admsmain_enumeration_obsolete_yes))\\\n";
  $codeH.="    adms_message_obsolete_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_error(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_error (adms_get_admsmain())==adms_admsmain_enumeration_error_yes))\\\n";
  $codeH.="    adms_message_error_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_fatal(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_fatal (adms_get_admsmain())==adms_admsmain_enumeration_fatal_yes))\\\n";
  $codeH.="    adms_message_fatal_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_info_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_info (adms_get_admsmain())==adms_admsmain_enumeration_info_yes))\\\n";
  $codeH.="    adms_message_info_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_usage_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_usage (adms_get_admsmain())==adms_admsmain_enumeration_usage_yes))\\\n";
  $codeH.="    adms_message_usage_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_verbose_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_verbose (adms_get_admsmain())==adms_admsmain_enumeration_verbose_yes))\\\n";
  $codeH.="    adms_message_verbose_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_debug_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_debug (adms_get_admsmain())==adms_admsmain_enumeration_debug_yes))\\\n";
  $codeH.="    adms_message_debug_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_hint_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_hint (adms_get_admsmain())==adms_admsmain_enumeration_hint_yes))\\\n";
  $codeH.="    adms_message_hint_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_warning_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_warning (adms_get_admsmain())==adms_admsmain_enumeration_warning_yes))\\\n";
  $codeH.="    adms_message_warning_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_obsolete_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_obsolete (adms_get_admsmain())==adms_admsmain_enumeration_obsolete_yes))\\\n";
  $codeH.="    adms_message_obsolete_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_error_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_error (adms_get_admsmain())==adms_admsmain_enumeration_error_yes))\\\n";
  $codeH.="    adms_message_error_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="#define adms_message_fatal_continue(...)\\\n";
  $codeH.="{\\\n";
  $codeH.="  if((adms_get_admsmain())\\\n";
  $codeH.="    &&(adms_admsmain_valueof_fatal (adms_get_admsmain())==adms_admsmain_enumeration_fatal_yes))\\\n";
  $codeH.="    adms_message_fatal_continue_impl(__VA_ARGS__);\\\n";
  $codeH.="}\n";
  $codeH.="\n";
  return ($codeH,$codeC);
}
sub mkLink_ADMSXML_TO_ADMSTRANSFORM
{
  my $codeH;
  my $codeC;
  $codeC.="/*-- link between adms.xml and admsttransform --*/\n";
  $codeC.="#include <admsObject/admsC.h>\n";

  foreach my$accessor qw(valueof isdefault)
  {
    my$ret;
    $ret="p_adms" if $accessor eq "valueof";
    $ret="int" if $accessor eq "isdefault";
    $codeH.="win32_interface $ret adms_adms_$accessor (p_admst myadmst,p_adms myitem,const char*mypath);\n";
    $codeC.="$ret adms_adms_$accessor (p_admst myadmst,p_adms myitem,const char*mypath)\n";
    $codeC.=q[{
  int pDataTypeName;
  if(myadmst->_datatypefamily==adms_admst_enumeration_datatypefamily_basictype)
    adms_message_fatal("basictype not allowed",adms_admst_getuid(myadmst))
  if(myadmst->_datatypename==adms_adms_enumeration_datatypename__star)
    pDataTypeName=myadmst->_item->_datatypename;
  else
    pDataTypeName=myadmst->_datatypename;
];
    foreach(@$myElementArray)
    {
      my$element=$_;
      my$elementName=$element->getAttribute("name");
      $codeC.="  if(pDataTypeName==adms_adms_enumeration_datatypename_$elementName)\n  {\n";
      my@attribute=&element_attribute($element);
      foreach(@attribute)
      {
        my$attribute_name=&attribute_name($_);
        my$attribute_datatypename=&attribute_datatypename($_);
        $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
        $codeC.="      return ($ret)".&buildsymbolvalueBis($_,$accessor)."(myitem);\n";
      }
      my$parent=$element;
      if($accessor ne "isdefault")
      {
        while($parent=&element_parent($parent))
        {
          my$attribute_name=$parent->getAttribute ("name");
          $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
          $codeC.="      return ($ret)".&buildsupersymbolname($element,$parent,$accessor)."(myitem);\n";
        }
      }
      $codeC.="  }\n";
    }
    $codeC.="  adms_message_fatal_continue(\"%s: missing node %s/%s\\n\",
      adms_admst_getuid(myadmst),adms_adms_tostring_datatypename_impl(myadmst->_admsttransform,pDataTypeName),mypath)\n";
    $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
    $codeC.="}\n";
  }

  $codeH.="win32_interface int adms_admsttransform_valueof_size (p_admst myadmst,const char*mypath);\n";
  $codeC.="int adms_admsttransform_valueof_size (p_admst myadmst,const char*mypath)\n";
  $codeC.="{";
  $codeC.=q[
  e_admst_datatypename pDataTypeName;
  if(myadmst->_datatypename==adms_adms_enumeration_datatypename__star)
    pDataTypeName=myadmst->_item->_datatypename;
  else
    pDataTypeName=myadmst->_datatypename;
];
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(pDataTypeName==adms_adms_enumeration_datatypename_$elementName)\n  {\n";
    my@attribute=&element_attribute($element);
    foreach(@attribute)
    {
      my$attribute_name=&attribute_name($_);
      my$attribute_datatypename=&attribute_datatypename($_);
      $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
      $codeC.="      return ".&buildsymbolvalue($_,"size").";\n";
    }
    ####my$parent=$element;
    ####while($parent=&element_parent($parent))
    ####{
    ####  my$attribute_name=$parent->getAttribute ("name");
    ####  $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
    ####  $codeC.="      return adms_adms_enumeration_datatypefamily_$attribute_name;\n";
    ####}
    $codeC.="  }\n";
  }
  $codeC.="  adms_message_fatal_continue(\"%s: missing node %s/%s\\n\",
    adms_admst_getuid(myadmst),adms_admst_tostring_size_impl(myadmst->_admsttransform,pDataTypeName),mypath)\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";

  $codeH.="win32_interface void adms_valueto (p_admst myadmst,const char*mypath,const p_adms myvalue);\n";
  $codeC.="void adms_valueto (p_admst myadmst,const char*mypath,const p_adms myvalue)\n";
  $codeC.="{";
  $codeC.=q[
  e_admst_datatypename pDataTypeName;
  if(myadmst->_datatypename==adms_adms_enumeration_datatypename__star)
    pDataTypeName=myadmst->_item->_datatypename;
  else
    pDataTypeName=myadmst->_datatypename;
];
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(pDataTypeName==adms_adms_enumeration_datatypename_$elementName)\n  {\n";
    my@attribute=&element_attribute($element);
    foreach(@attribute)
    {
      my$cast;
      my$attribute_name=&attribute_name($_);
      my$attribute_datatypename=&attribute_datatypename($_);
      my$attribute_size=&attribute_size($_);
      if($attribute_datatypename eq "*")
      { $cast="myvalue";}
      elsif($attribute_datatypename eq "string")
      { $cast="(char*)myvalue";}
      elsif($attribute_datatypename eq "integer")
      { $cast="GPOINTER_TO_INT(myvalue)";}
      elsif($attribute_datatypename eq "enumeration")
      { $cast="GPOINTER_TO_INT(myvalue)";}
      else
      {$cast="(p_$attribute_datatypename)myvalue";}
      if($attribute_size eq "list")
      {$cast="(GSList*)myvalue";}
      $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
      $codeC.="      {".&buildsymbolvalueBis($_,"valueof")."(myadmst->_item)=$cast; return;}\n";
    }
    ####my$parent=$element;
    ####while($parent=&element_parent($parent))
    ####{
    ####  my$attribute_name=$parent->getAttribute ("name");
    ####  $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
    ####  $codeC.="      return adms_adms_enumeration_datatypefamily_$attribute_name;\n";
    ####}
    $codeC.="  }\n";
  }
  $codeC.="  adms_message_fatal_continue(\"%s: missing node %s/%s\\n\",
    adms_admst_getuid(myadmst),adms_admst_tostring_size_impl(myadmst->_admsttransform,pDataTypeName),mypath)\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";

  $codeH.="win32_interface int adms_admsttransform_valueof_datatypefamily (p_admst myadmst,const char*mypath);\n";
  $codeC.="int adms_admsttransform_valueof_datatypefamily (p_admst myadmst,const char*mypath)\n";
  $codeC.="{";
  $codeC.=q[
  e_admst_datatypename pDataTypeName;
  if(myadmst->_datatypename==adms_adms_enumeration_datatypename__star)
    pDataTypeName=myadmst->_item->_datatypename;
  else
    pDataTypeName=myadmst->_datatypename;
];
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(pDataTypeName==adms_adms_enumeration_datatypename_$elementName)\n  {\n";
    my@attribute=&element_attribute($element);
    foreach(@attribute)
    {
      my$attribute_name=&attribute_name($_);
      my$attribute_datatypename=&attribute_datatypename($_);
      $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
      $codeC.="      return ".&buildsymbolvalue($_,"datatypefamily").";\n";
    }
    ####my$parent=$element;
    ####while($parent=&element_parent($parent))
    ####{
    ####  my$attribute_name=$parent->getAttribute ("name");
    ####  $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
    ####  $codeC.="      return adms_adms_enumeration_datatypefamily_$attribute_name;\n";
    ####}
    $codeC.="  }\n";
  }
  $codeC.="  adms_message_fatal_continue(\"%s: missing node %s/%s\\n\",
    adms_admst_getuid(myadmst),adms_admst_tostring_datatypefamily_impl(myadmst->_admsttransform,pDataTypeName),mypath)\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";

  $codeH.="win32_interface int adms_admsttransform_valueof_datatypename (p_admst myadmst,const char*mypath);\n";
  $codeC.="int adms_admsttransform_valueof_datatypename (p_admst myadmst,const char*mypath)\n";
  $codeC.="{";
  $codeC.=q[
  e_admst_datatypename pDataTypeName;
  if(myadmst->_datatypename==adms_adms_enumeration_datatypename__star)
    pDataTypeName=myadmst->_item->_datatypename;
  else
    pDataTypeName=myadmst->_datatypename;
];
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(pDataTypeName==adms_adms_enumeration_datatypename_$elementName)\n  {\n";
    my@attribute=&element_attribute($element);
    foreach(@attribute)
    {
      my$attribute_name=&attribute_name($_);
      my$attribute_datatypename=&attribute_datatypename($_);
      $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
      $codeC.="      return ".&buildsymbolvalue($_,"datatypename").";\n";
    }
    my$parent=$element;
    while($parent=&element_parent($parent))
    {
      my$attribute_name=$parent->getAttribute ("name");
      $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n";
      $codeC.="      return adms_adms_enumeration_datatypename_$attribute_name;\n";
    }
    $codeC.="  }\n";
  }
  $codeC.="  adms_message_fatal_continue(\"%s: missing node %s/%s\\n\",
    adms_admst_getuid(myadmst),adms_adms_tostring_datatypename_impl(myadmst->_admsttransform,pDataTypeName),mypath)\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";

  $codeH.="win32_interface void adms_admsttransform_linkto_admsxml (p_admst myadmst);\n";
  $codeC.="void adms_admsttransform_linkto_admsxml (p_admst myadmst)\n";
  $codeC.="{\n";
  $codeC.=q[
  const char*mypath=myadmst->_attributename;
  int pDataTypeName;
  if(myadmst->_previous->_datatypefamily==adms_admst_enumeration_datatypefamily_basictype)
    adms_message_fatal("basictype not allowed",adms_admst_getuid(myadmst->_previous))
  if(myadmst->_previous->_datatypename==adms_adms_enumeration_datatypename__star)
    pDataTypeName=myadmst->_previous->_item->_datatypename;
  else
    pDataTypeName=myadmst->_previous->_datatypename;
];
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(pDataTypeName==adms_adms_enumeration_datatypename_$elementName)\n  {\n";
    my@attribute=&element_attribute($element);
    foreach(@attribute)
    {
      my$attribute_name=&attribute_name($_);
      my$attribute_datatypename=&attribute_datatypename($_);
      $codeC.="    if (!strcmp(mypath,\"$attribute_name\"))\n    {\n";
      foreach my $property (qw(valueto))
      {
        $codeC.="      myadmst->_$property=".&buildsymbolvalue($_,$property).";\n";
      }
      foreach my $property (qw(datatypename size datatypefamily))
      {
        $codeC.="      myadmst->_$property=".&buildsymbolvalue($_,$property).";\n";
      }
      if($attribute_datatypename eq "enumeration")
      {
        foreach my $property (qw(fromstring tostring))
        {
          $codeC.="      myadmst->_$property=".&buildsymbolvalue($_,$property).";\n";
        }
      }
      $codeC.="      return;\n";
      $codeC.="    }\n";
    }
    my$parent=$element;
    while($parent=&element_parent($parent))
    {
      my$attribute_name=$parent->getAttribute ("name");
      $codeC.="    if(!strcmp(mypath,\"$attribute_name\"))\n    {\n";
      $codeC.="      myadmst->_datatypename=adms_adms_enumeration_datatypename_$attribute_name;\n";
      my$valueof=&buildsupersymbolname($element,$parent,"valueof");
      $codeC.="      myadmst->_valueto=NULL;\n";
      $codeC.="      return;\n";
      $codeC.="    }\n";
    }
    $codeC.="  }\n";
  }
  $codeC.="  adms_message_fatal_continue(\"%s: missing node %s/%s\\n\",
    adms_admst_getuid(myadmst),adms_adms_tostring_datatypename_impl(myadmst->_admsttransform,pDataTypeName),mypath)\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";
  return ($codeH,$codeC);
}

sub mkGet_UID
{
  my $codeH;
  my $codeC;
  $codeH.="win32_interface const char*h(p_adms myitem);\n";
  $codeC.="const char*h(p_adms myitem)\n";
  $codeC.="{\n";
  $codeC.="  int dataTypeName=myitem->_datatypename;\n";
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(dataTypeName==adms_adms_enumeration_datatypename_$elementName)\n  {\n" if($elementName ne "adms");
    $codeC.="    return adms_${elementName}_getuid(adms_$elementName(myitem));\n  }\n" if($elementName ne "adms");
  }
  $codeC.="  adms_message_fatal_continue(\"%s:%i: internal error - should not be reached\\n\",__FILE__,__LINE__)\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";
  return ($codeH,$codeC);
}
sub mkGet_NEW
{
  my $codeH;
  my $codeC;
  $codeH.="win32_interface const p_new adms_valueof_new(p_admsttransform myadmsttransform);\n";
  $codeC.="const p_new adms_valueof_new(p_admsttransform myadmsttransform)\n";
  $codeC.="{\n";
  $codeC.="  char*dataTypeName=myadmsttransform->_datatype;\n";
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(!strcmp(dataTypeName,\"$elementName\"))\n" if($elementName ne "adms");
    $codeC.="    return adms_${elementName}_new_impl;\n" if($elementName ne "adms");
  }
  $codeC.="  adms_message_fatal_continue(\"%s: missing element %s\\n\",h(adms_adms(myadmsttransform)),dataTypeName)\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";
  return ($codeH,$codeC);
}
sub mkGet_CMP
{
  my $codeH;
  my $codeC;
  $codeH.="win32_interface const p_cmp adms_valueof_cmp(p_admst myadmst);\n";
  $codeC.="const p_cmp adms_valueof_cmp(p_admst myadmst)\n";
  $codeC.="{\n";
  $codeC.="  int dataTypeName=myadmst->_datatypename;\n";
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(dataTypeName==adms_admst_enumeration_datatypename_$elementName)\n";
    $codeC.="    return (p_cmp) adms_${elementName}_cmp;\n";
  }
  $codeC.="  adms_message_fatal_continue(\"%s: missing element %s\\n\",h(adms_adms(myadmst)),adms_adms_tostring_datatypename_impl(myadmst->_admsttransform,dataTypeName))\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";
  return ($codeH,$codeC);
}
sub mkGet_ALLCHILDREN
{
  my $codeH;
  my $codeC;
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    my@attribute=&element_attribute($element);
    my@Attribute;
    foreach(@attribute)
    {
      my$attribute_name=&attribute_name($_);
      push @Attribute, $attribute_name;
    }
    my$size=scalar(@Attribute)+1;
    $codeC.="const char*adms_element_${elementName}_allattribute[$size]={\"".(join"\",\"",@Attribute)."\",NULL};\n";
  }
  $codeH.="win32_interface const char** adms_admstpath_valueof_allchildren(const p_admst myadmst);\n";
  $codeC.="const char** adms_admstpath_valueof_allchildren(const p_admst myadmst)\n";
  $codeC.="{\n";
  $codeC.="  int dataTypeName=myadmst->_datatypename;\n";
  $codeC.="/*fixme: can be list*/\n";
  $codeC.="  if(dataTypeName==adms_admst_enumeration_datatypename__star)\n";
  $codeC.="    dataTypeName=myadmst->_item->_datatypename;\n";
  foreach(@$myElementArray)
  {
    my$element=$_;
    my$elementName=$element->getAttribute("name");
    $codeC.="  if(dataTypeName==adms_admst_enumeration_datatypename_$elementName)\n";
    $codeC.="    return adms_element_${elementName}_allattribute;\n";
  }
  $codeC.="  adms_message_fatal_continue(\"%s: missing element %s\\n\",h(adms_adms(myadmst)),adms_adms_tostring_datatypename_impl(myadmst->_admsttransform,dataTypeName))\n";
  $codeC.="  adms_message_fatal(\"see http://mot-adms.sourceforge.net/xml-files/adms/adms.xml\")\n";
  $codeC.="}\n";
  return ($codeH,$codeC);
}
